package org.kar.karanage;

import java.net.URI;

import jakarta.ws.rs.core.UriBuilder;

import org.glassfish.grizzly.http.server.HttpServer;
import org.glassfish.jersey.grizzly2.httpserver.GrizzlyHttpServerFactory;
import org.glassfish.jersey.jackson.JacksonFeature;
import org.glassfish.jersey.media.multipart.MultiPartFeature;
import org.glassfish.jersey.server.ResourceConfig;
import org.kar.karanage.api.Front;
import org.kar.karanage.api.HealthCheck;
import org.kar.karanage.api.LogResource;
import org.kar.karanage.api.StateHistoryResource;
import org.kar.karanage.api.StateResource;
import org.kar.karanage.api.UserResource;
import org.kar.karanage.filter.KaranageAuthenticationFilter;
import org.kar.karanage.migration.Initialization;
import org.kar.archidata.GlobalConfiguration;
import org.kar.archidata.UpdateJwtPublicKey;
import org.kar.archidata.catcher.ExceptionCatcher;
import org.kar.archidata.catcher.FailExceptionCatcher;
import org.kar.archidata.catcher.InputExceptionCatcher;
import org.kar.archidata.catcher.SystemExceptionCatcher;
import org.kar.archidata.filter.CORSFilter;
import org.kar.archidata.filter.OptionFilter;
import org.kar.archidata.migration.MigrationEngine;
import org.kar.archidata.util.ConfigBaseVariable;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

public class WebLauncher {
	final static Logger LOGGER = LoggerFactory.getLogger(WebLauncher.class);
	protected UpdateJwtPublicKey keyUpdater = null; 
	public WebLauncher() {
		ConfigBaseVariable.bdDatabase = "karanage";
	}
	
    private static URI getBaseURI() {
        return UriBuilder.fromUri(ConfigBaseVariable.getlocalAddress()).build();
    }

	public void migrateDB() throws Exception {
		WebLauncher.LOGGER.info("Create migration engine");
		MigrationEngine migrationEngine = new MigrationEngine();
		WebLauncher.LOGGER.info("Add initialization");
		migrationEngine.setInit(new Initialization());
		WebLauncher.LOGGER.info("Add migration since last version");
		// NOTHING for now
		WebLauncher.LOGGER.info("Migrate the DB [START]");
		migrationEngine.migrate(GlobalConfiguration.dbConfig);
		WebLauncher.LOGGER.info("Migrate the DB [STOP]");
	}

	public static void main(String[] args) throws Exception {
		WebLauncher.LOGGER.info("[START] application wake UP");
		WebLauncher launcher = new WebLauncher();
		launcher.migrateDB();
		
		launcher.process();
		WebLauncher.LOGGER.info("end-configure the server & wait finish process:");
		Thread.currentThread().join();
		WebLauncher.LOGGER.info("STOP Key updater");
		launcher.stopOther();
		WebLauncher.LOGGER.info("STOP the REST server:");
	}
    	

    public void process() throws InterruptedException {
		// ===================================================================
		// Configure resources
		// ===================================================================
        ResourceConfig rc = new ResourceConfig();
        
        // add multipart models ..
        rc.register(MultiPartFeature.class);
        // global authentication system
        rc.register(OptionFilter.class);
        // remove cors ==> all time called by an other system...
        rc.register(CORSFilter.class);
        // global authentication system
        rc.register(KaranageAuthenticationFilter.class);
        // register exception catcher
        rc.register(InputExceptionCatcher.class);
        rc.register(SystemExceptionCatcher.class);
        rc.register(FailExceptionCatcher.class);
        rc.register(ExceptionCatcher.class);
        
        // add default resource:
        rc.register(UserResource.class);
        rc.register(StateResource.class);
        rc.register(StateHistoryResource.class);
        rc.register(LogResource.class);
        
        
        rc.register(HealthCheck.class);
        rc.register(Front.class);
        
        // add jackson to be discovenr when we are ins standalone server
        rc.register(JacksonFeature.class);
        // enable this to show low level request
        //rc.property(LoggingFeature.LOGGING_FEATURE_LOGGER_LEVEL_SERVER, Level.WARNING.getName());

		//System.out.println("Connect on the BDD:");
		//System.out.println("    getDBHost: '" + ConfigVariable.getDBHost() + "'");
		//System.out.println("    getDBPort: '" + ConfigVariable.getDBPort() + "'");
		//System.out.println("    getDBLogin: '" + ConfigVariable.getDBLogin() + "'");
		//System.out.println("    getDBPassword: '" + ConfigVariable.getDBPassword() + "'");
		//System.out.println("    getDBName: '" + ConfigVariable.getDBName() + "'");
		System.out.println(" ==> " + GlobalConfiguration.dbConfig);
		System.out.println("OAuth service " + getBaseURI());
		HttpServer server = GrizzlyHttpServerFactory.createHttpServer(getBaseURI(), rc);
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
			@Override
			public void run() {
				System.out.println("Stopping server..");
				server.shutdownNow();
			}
		}, "shutdownHook"));

		// ===================================================================
		// start periodic update of the token ...
		// ===================================================================
		this.keyUpdater = new UpdateJwtPublicKey();
		this.keyUpdater.start();

 		// ===================================================================
		// run JERSEY
 		// ===================================================================
		try {
			server.start();
			System.out.println("Jersey app started at " + getBaseURI());
		} catch (Exception e) {
			System.out.println("There was an error while starting Grizzly HTTP server.");
			e.printStackTrace();
		}
    }
    
    public void stopOther() {
		keyUpdater.kill();
		try {
			keyUpdater.join(4000, 0);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
}
